{--(02 aug 2000)----------------------------------------------------------------

    TTrayIcon Delphi Class
    incapsulating class for using the systemtray.

    Get started :
      You create an instance ( x := Ttrayicon.create )
      You add icons and link it with a unique ID. (x.add(ID,icon) )
      You hook up the events which tells you if a user clicks on a given icon.
         ( X.OnUpdateIcon = MyMethod )

    Written by Lars Peter Christiansen
    home1.stofanet.dk/nitezhifter

    This product is FreeWare and can be freely distributed. It cannot be used
    for commercial purposes without the author's approval!

-------------------------------------------------------------------------------}
unit systrayclass;

interface
 uses windows,controls,messages,shellapi,graphics,commctrl,sysutils,forms;

   const WM_TRAYICON = WM_USER+100;  {Message being passed to tray icon}


   Type TTrayIconState = record
     ID   : Word;
     Icon : Ticon;
     Flashes:boolean;
   end;

   Type Ttrayicon = class
     private
        Ficondata : TNotifyIconData; {Structure needed for tray icon}
        Fwndproc : TwndMethod; // saves original messagehandler
        Fcontrol : Tcontrol;
        Fvisible : Boolean;
        Ficons : array of Ttrayiconstate;
        Flastindex : integer;
        Ficonindex:integer;
        fflash :boolean;
        procedure MyProc(var Message:Tmessage);
        procedure SetVisible(const Value: boolean);
        procedure MakeMouseEvent(mb:TmouseButton;ID:word);
        function IDtoIndex(ID:word) : Integer;
    function GetIconID: Word;
    function GetHint: string;
    procedure SetHint(const Value: string);
      public
        OnMouseDown : Procedure(Mousebutton:Tmousebutton;IconID:Word) of object;
        OnUpdateIcon: Procedure(Icon:Ticon) of object;
        Constructor create (aWinControl:Twincontrol);
        destructor Destroy;override;
        function ChangeIcon(iconID:word):boolean;
        procedure AddIcon(ID:word;Icon:Hicon);
        procedure Last;
        function IDtoIcon(id:word) : Ticon;
        function RemoveIcon(iconID:word):boolean;
        procedure IncTime;
        property Visible :boolean read Fvisible write SetVisible;
        property Flashing : boolean read Fflash;
        property IconID : Word read GetIconID;
        Property Hint : string read GetHint write SetHint;
        Procedure update;
   end;
implementation
{ TNLtrayicon }

procedure Ttrayicon.AddIcon(ID:word;Icon:Hicon);
var
l:integeR;
begin
  L :=length(Ficons);
  setlength(Ficons,L+1);

  Ficons[high(Ficons)].icon := Ticon.create;
  Ficons[high(Ficons)].icon.handle :=icon;
  Ficons[high(Ficons)].id:=id;
  Ficons[high(Ficons)].flashes:=false;
  if L=0 then Ficonindex:=0;
end;



function Ttrayicon.ChangeIcon(iconID: word): boolean;
var i :integeR;
begin
  i := IDtoIndex(iconID);
  if i< 0 then Result:=false
  else begin
//       if FiconIndex<>i then
         begin
           FiconIndex:=i;
           Update;
         end;
       result:=true;
       end;

end;

constructor Ttrayicon.create(aWinControl: Twincontrol);
begin
Fvisible:=false;
   {This is the tray icon structure being created}
   FiconData.cbSize := SizeOf(FiconData);
   FiconData.Wnd := awincontrol.Handle;
   FiconData.uID := 1;
   FiconData.uFlags := NIF_ICON or NIF_TIP or NIF_MESSAGE;
   FiconData.uCallbackMessage := WM_TRAYICON; {User defined message}

FwndProc := awincontrol.WindowProc;
awincontrol.WindowProc:=myproc;
Ficonindex:=-1;
setlength(Ficons,0);
Fcontrol := aWincontrol;
end;

destructor Ttrayicon.Destroy;
begin
Fcontrol.windowproc:=Fwndproc;
Shell_NotifyIcon(NIM_DELETE,@Ficondata);
inherited;

end;


function Ttrayicon.GetHint: string;
begin
result := strPas(@Ficondata.szTip);
end;

function Ttrayicon.GetIconID: Word;
begin
result :=Ficons[Ficonindex].id
end;

function Ttrayicon.IDtoIcon(id:word): Ticon;
var I:integer;
begin
 I:=idtoindex(id);
 if i>-1 then result := Ficons[i].icon
         else result := nil;
end;

function Ttrayicon.IDtoIndex(ID: word): Integer;
var H,i:integer;
begin
  result:=-1;
  H := high(Ficons);
  if H>0 then FlastIndex :=ficonindex;
  for i:=0 to H do
      if ID=Ficons[i].id then
        begin
          result:=i;
          break;
        end;
end;

procedure Ttrayicon.IncTime;
begin
  if Fflash then
    begin
      if Ficondata.hIcon= Ficons[ficonindex].icon.handle
        then Ficondata.hIcon:= Ficons[flastindex].icon.handle
        else Ficondata.hIcon:= Ficons[ficonindex].icon.handle;
      Shell_notifyicon(NIM_Modify,@Ficondata);
   if assigned(OnUpdateIcon) then onupdateicon(Ficons[FiconIndex].icon);
    end;
end;

procedure Ttrayicon.Last;
begin
  Ficonindex:=flastindex;
  update;
end;

procedure Ttrayicon.MakeMouseEvent(mb: TmouseButton; ID: word);
begin
    if assigned(OnMouseDown) then Onmousedown(mb,ID);
end;

procedure Ttrayicon.MyProc(var Message: Tmessage);
begin
FwndProc(message);
  CAse message.Msg of
  WM_TRAYICON:
    begin
      Case(message.LParam) of
        WM_LBUTTONDOWN : makemouseevent(mbleft,Ficons[ficonindex].id);
        WM_RBUTTONDOWN : makemouseevent(mbright,Ficons[ficonindex].id);
      end;
    end;
  end;
end;

function Ttrayicon.RemoveIcon(iconID: word):boolean;
begin
result:=false; // implement later...
end;

procedure Ttrayicon.SetHint(const Value: string);
begin
strCopy(@Ficondata.szTip,pchar(value) );
if visible then Shell_notifyicon(NIM_Modify,@Ficondata);
end;

procedure Ttrayicon.SetVisible(const Value: boolean);
begin
  if value then
    begin if Shell_NotifyIcon(NIM_ADD,@Ficondata) then Fvisible:=true;end
  else
    begin if Shell_NotifyIcon(NIM_DELETE,@Ficondata) then Fvisible:=false;end;
end;


procedure Ttrayicon.update;
begin

  Ficondata.hicon:= Ficons[FiconIndex].icon.handle;
  Shell_notifyicon(NIM_Modify,@Ficondata);
  FFlash:=Ficons[FiconIndex].flashes;
  if assigned(OnUpdateIcon) then
     onupdateicon(Ficons[FiconIndex].icon);
end;

end.
